@@ -1,6 +1,6 @@
-from uuid import UUID
-from typing import Union, List
+from typing import List
 
+from bleak.backends.bluezdbus.utils import extract_service_handle_from_path
 from bleak.backends.service import BleakGATTService
 from bleak.backends.bluezdbus.characteristic import BleakGATTCharacteristicBlueZDBus
 
@@ -12,12 +12,18 @@ class BleakGATTServiceBlueZDBus(BleakGATTService):
         super().__init__(obj)
         self.__characteristics = []
         self.__path = path
+        self.__handle = extract_service_handle_from_path(path)
 
     @property
     def uuid(self) -> str:
         """The UUID to this service"""
         return self.obj["UUID"]
 
+    @property
+    def handle(self) -> str:
+        """The integer handle of this service"""
+        return self.__handle
+
     @property
     def characteristics(self) -> List[BleakGATTCharacteristicBlueZDBus]:
         """List of characteristics for this service"""
