@@ -6,8 +6,10 @@ Created on 2018-04-23 by hbldh <henrik.blidh@nedomkull.com>
 
 """
 import abc
+import asyncio
 import uuid
 from typing import Callable, Union
+from warnings import warn
 
 from bleak.backends.service import BleakGATTServiceCollection
 from bleak.backends.characteristic import BleakGATTCharacteristic
@@ -118,8 +120,8 @@ class BaseBleakClient(abc.ABC):
         """Unpair with the peripheral."""
         raise NotImplementedError()
 
-    @abc.abstractmethod
-    async def is_connected(self) -> bool:
+    @abc.abstractproperty
+    def is_connected(self) -> bool:
         """Check connection status between this client and the server.
 
         Returns:
@@ -128,10 +130,32 @@ class BaseBleakClient(abc.ABC):
         """
         raise NotImplementedError()
 
+    class _DeprecatedIsConnectedReturn:
+        """Wrapper for ``is_connected`` return value to provide deprecation warning."""
+
+        def __init__(self, value: bool):
+            self._value = value
+
+        def __bool__(self):
+            return self._value
+
+        def __call__(self) -> bool:
+            warn(
+                "is_connected has been changed to a property. Calling it as an async method will be removed in a future version",
+                FutureWarning,
+                stacklevel=2,
+            )
+            f = asyncio.Future()
+            f.set_result(self._value)
+            return f
+
+        def __repr__(self) -> str:
+            return repr(self._value)
+
     # GATT services methods
 
     @abc.abstractmethod
-    async def get_services(self) -> BleakGATTServiceCollection:
+    async def get_services(self, **kwargs) -> BleakGATTServiceCollection:
         """Get all services registered for this GATT server.
 
         Returns:
