@@ -21,9 +21,6 @@ class BleakGATTDescriptorCoreBluetooth(BleakGATTDescriptor):
         self.__characteristic_uuid = characteristic_uuid
         self.__characteristic_handle = characteristic_handle
 
-    def __str__(self):
-        return "{0}: (Handle: {1})".format(self.uuid, self.handle)
-
     @property
     def characteristic_handle(self) -> int:
         """handle for the characteristic that this descriptor belongs to"""
