@@ -87,7 +87,7 @@ class BleakScannerCoreBluetooth(BaseBleakScanner):
         except Exception as e:
             logger.warning("stopScan method could not be called: {0}".format(e))
 
-    async def set_scanning_filter(self, **kwargs):
+    def set_scanning_filter(self, **kwargs):
         """Set scanning filter for the scanner.
 
         .. note::
@@ -113,6 +113,7 @@ class BleakScannerCoreBluetooth(BaseBleakScanner):
             address = peripheral.identifier().UUIDString()
             name = peripheral.name() or "Unknown"
             details = peripheral
+            rssi = self._manager.devices[address].rssi
 
             advertisementData = self._identifiers[peripheral.identifier()]
             manufacturer_binary_data = advertisementData.get(
@@ -131,13 +132,20 @@ class BleakScannerCoreBluetooth(BaseBleakScanner):
                 for u in advertisementData.get("kCBAdvDataServiceUUIDs", [])
             ]
 
+            service_data = {}
+            adv_service_data = advertisementData.get("kCBAdvDataServiceData", [])
+            for u in adv_service_data:
+                service_data[cb_uuid_to_str(u)] = bytes(adv_service_data[u])
+
             found.append(
                 BLEDevice(
                     address,
                     name,
                     details,
+                    rssi=rssi,
                     uuids=uuids,
                     manufacturer_data=manufacturer_data,
+                    service_data=service_data,
                     delegate=self._manager.central_manager.delegate(),
                 )
             )
