@@ -18,6 +18,11 @@ class BleakGATTServiceDotNet(BleakGATTService):
             # BleakGATTCharacteristicDotNet(c) for c in obj.GetAllCharacteristics()
         ]
 
+    @property
+    def handle(self) -> str:
+        """The handle of this service"""
+        return int(self.obj.AttributeHandle)
+
     @property
     def uuid(self) -> str:
         """UUID for this service."""
